/*
 * servdemo.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <vdr/plugin.h>

// include header file from wirbelscan plugin
#include "../wirbelscan/wirbelscan_services.h"

using namespace WIRBELSCAN_SERVICE;

static const char *VERSION        = "0.0.1";
static const char *DESCRIPTION    = "wirbelscan service demo";
static const char *MAINMENUENTRY  = "Servdemo";

class cPluginServdemo : public cPlugin {
private:

public:
  cPluginServdemo(void) {}
  virtual ~cPluginServdemo() {}
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return DESCRIPTION; }
  virtual const char *CommandLineHelp(void) { return NULL; }
  virtual bool ProcessArgs(int argc, char *argv[]) { return true; }
  virtual bool Initialize(void) { return true; }
  virtual bool Start(void) { return true; }
  virtual void Stop(void) { }
  virtual void Housekeeping(void) { }
  virtual void MainThreadHook(void) { }
  virtual cString Active(void) { return NULL; }
  virtual time_t WakeupTime(void) { return 0; }
  virtual const char *MainMenuEntry(void) { return MAINMENUENTRY; }
  virtual cOsdObject *MainMenuAction(void) { return NULL; }
  virtual cMenuSetupPage *SetupMenu(void) { return NULL; }
  virtual bool SetupParse(const char *Name, const char *Value) { return false; }
  virtual bool Service(const char *Id, void *Data = NULL) { return false; }
  virtual const char **SVDRPHelpPages(void);
  virtual cString SVDRPCommand(const char *Command, const char *Option, int &ReplyCode);
  };


const char **cPluginServdemo::SVDRPHelpPages(void)
{
  static const char * SVDRHelp[] = {
    "START\n"
    "    send wirbelscan command 'start scan'",
    "STOP\n"
    "    send wirbelscan command 'stop scan'",
    "GETS\n"
    "    get wirbelscans setup data",
    "SETS <verb:log:type:inv_t:inv_c:srate:qam:cidx:sidx:flags:atsc>\n"
    "    set wirbelscans setup data\n"
    "    verb   verbostity (0..5)\n"
    "    log    logfile (0=OFF, 1=stdout, 2=syslog)\n"
    "    type   scan type\n"
    "           (0=DVB-T, 1=DVB-C, 2=DVB-S/S2, 3=pvrinput, 4=pvrinput FM, 5=ATSC)\n"
    "    inv_t  DVB-T inversion\n"
    "           (0=AUTO/OFF, 1=AUTO/ON)\n"
    "    inv_c  DVB-C inversion\n"
    "           (0=AUTO/OFF, 1=AUTO/ON)\n"
    "    srate  DVB-C Symbolrate (0..15)\n"
    "    qam    DVB-C modulation\n"
    "           (0=AUTO, 1=QAM64, 2=QAM128, 3=QAM256, 4=ALL)\n"
    "    cidx   country list index\n"
    "    sidx   satellite list index\n"
    "    flags  bitwise flag of\n"
    "           TV=1, RADIO=2, FTA=4, SCRAMBLED=8, HDTV=16"
    "    atsc   ATSC scan type\n"
    "           (0=VSB, 1=QAM, 2=VSB+QAM)\n",
    "STORE\n"
    "    trigger wirbelscan to store it's current settings",
    "QUERY\n"
    "    query wirbelscan plugin version and command api",
    "LSTS\n"
    "    list satellites",
    "LSTC\n"
    "    list countries",
    "STAT\n"
    "    query wirbelscan status",
    NULL
    };
  return SVDRHelp;
}

void SendCmd(cWirbelscanCmd * cmd)
{
 cPlugin *wirbelscan = cPluginManager::GetPlugin("wirbelscan");
 char * request;
 // "wirbelscan_DoCmd#<VERSION>"
 asprintf(&request, "%s%s", SPlugin, SCommand);

 if (wirbelscan) {
     if (wirbelscan->Service(request, cmd) == true) {
         if (! cmd->replycode)
             esyslog("%s failed.\n",
                cmd->cmd == CmdStartScan?"CmdStartScan":cmd->cmd == CmdStopScan?"CmdStopScan":"CmdStore");
         }
     else
         esyslog("wirbelscan was found, but this plugin version doesnt "
                "support the plugin service api.\n");
     }
 else
     esyslog("wirbelscan not found.\n");
 free(request);
}

bool QueryVersions(cWirbelscanInfo * info)
{
 cPlugin *wirbelscan = cPluginManager::GetPlugin("wirbelscan");
 char * s;

 if (wirbelscan) {
     bool ret;
     // "wirbelscan_GetVersion"
     asprintf(&s, "%s%s", SPlugin, SInfo); 
     ret = wirbelscan->Service(s, info);
     free(s);
     return ret;
     }
 else
     esyslog("wirbelscan not found.\n");
 return false;
}

cString ListSatellites()
{
 cPlugin *wirbelscan = cPluginManager::GetPlugin("wirbelscan");
 char * s;
 cString ret = cString::sprintf("%5s  %-9s  %-6s\n",
                       "id", "shortname", "fullname");

 if (wirbelscan) {
     // "wirbelscan_GetSat#<VERSION>"
     asprintf(&s, "%sGet%s", SPlugin, SSat);
     if (wirbelscan->Service(s, NULL) == true) {
         SListItem * buffer;

         cPreAllocBuffer b = {0, 0, NULL};
         wirbelscan->Service(s, &b);                               // query buffer size.
         buffer = (SListItem*) malloc(b.size * sizeof(SListItem)); // now, allocate memory.
         b.buffer = buffer;                                        // assign buffer
         wirbelscan->Service(s, &b);                               // fill buffer with values.
         free(s);
         for (uint i = 0; i < b.count; i++) {
             ret = cString::sprintf("%s%5d  %-9s  %-6s\n", *ret,
                   b.buffer[i].id, b.buffer[i].short_name, b.buffer[i].full_name);
             }
         return ret;
         }
     else {
         free(s);
         return cString::sprintf("This plugin version doesnt support requested api.");
         }
     }
 else
     return cString::sprintf("wirbelscan not found.\n");
}

cString ListCountries()
{
 cPlugin *wirbelscan = cPluginManager::GetPlugin("wirbelscan");
 char * s;
 cString ret = cString::sprintf("%5s  %-9s  %-6s\n",
                       "id", "shortname", "fullname");

 if (wirbelscan) {
     // ""wirbelscan_GetCountry#<VERSION>"
     asprintf(&s, "%sGet%s", SPlugin, SCountry); 
     if (wirbelscan->Service(s, NULL) == true) {
         SListItem * buffer;

         cPreAllocBuffer b = {0, 0, NULL};
         wirbelscan->Service(s, &b);                               // query buffer size.
         buffer = (SListItem*) malloc(b.size * sizeof(SListItem)); // now, allocate memory.
         b.buffer = buffer;                                        // assign buffer
         wirbelscan->Service(s, &b);                               // fill buffer with values.
         free(s);
         for (uint i = 0; i < b.count; i++) {
             ret = cString::sprintf("%s%5d  %-9s  %-6s\n", *ret,
                   b.buffer[i].id, b.buffer[i].short_name, b.buffer[i].full_name);
             }
         return ret;
         }
     else {
         free(s);
         return cString::sprintf("This plugin version doesnt support requested api.");
         }
     }
 else
     return cString::sprintf("wirbelscan not found.\n");
}

cString Status()
{
 cPlugin *wirbelscan = cPluginManager::GetPlugin("wirbelscan");
 char * s;
 cString ret = "";

 if (wirbelscan) {
     cWirbelscanStatus status;
     // "wirbelscan_Status#<VERSION>"
     asprintf(&s, "%sGet%s", SPlugin, SStatus); 
     if (wirbelscan->Service(s, &status) == true) {
         ret = cString::sprintf(  "status         : %s\n",
                  status.status == StatusUnknown? "unknown":
                  status.status == StatusScanning? "scanning":
                  status.status == StatusStopped? "stopped":
                  "busy");
         ret = cString::sprintf("%sdevice         : %s\n", *ret,
                  status.curr_device? status.curr_device:"(NULL pointer)");
         ret = cString::sprintf("%stransponder    : %s\n", *ret,
                  status.transponder? status.transponder:"(NULL pointer)");
         ret = cString::sprintf("%sstrength       : %u (%u%%)\n", *ret, status.strength, (int) (status.strength/655.35));
         ret = cString::sprintf("%sprogress       : %u%%\n", *ret, status.progress);
         ret = cString::sprintf("%sall channels   : %u\n", *ret, status.numChannels);
         ret = cString::sprintf("%snew channels   : %u", *ret, status.newChannels);
         return ret;
         }
     else {
         esyslog("wirbelscan was found, but this plugin version doesnt "
                "support the plugin service api.\n");
         free(s);
         return cString::sprintf("This plugin version doesnt support requested api.");
         }
     }
 else
     return cString::sprintf("wirbelscan not found.\n");
}

cString setup(cWirbelscanScanSetup * ScanSetup)
{
 cPlugin *wirbelscan = cPluginManager::GetPlugin("wirbelscan");
 char * s;
 cString ret;

 if (wirbelscan) {
     if (! ScanSetup) {
         cWirbelscanScanSetup data;
         // "wirbelscan_GetSetup#<VERSION>"
         asprintf(&s, "%sGet%s", SPlugin, SSetup);
         if (wirbelscan->Service(s, &data) == true) {
            free(s);
            ret = cString::sprintf("--current settings--\n");
            ret = cString::sprintf("%sverbosity      : %hu\n", *ret, data.verbosity);
            ret = cString::sprintf("%slogFile        : %hu\n", *ret, data.logFile);
            ret = cString::sprintf("%sDVB_type       : %hu\n", *ret, data.DVB_Type);
            ret = cString::sprintf("%sDVBT_Inversion : %hu\n", *ret, data.DVBT_Inversion);
            ret = cString::sprintf("%sDVBC_Inversion : %hu\n", *ret, data.DVBC_Inversion);
            ret = cString::sprintf("%sDVBC_Symbolrate: %hu\n", *ret, data.DVBC_Symbolrate);
            ret = cString::sprintf("%sDVBC_QAM       : %hu\n", *ret, data.DVBC_QAM);
            ret = cString::sprintf("%sCountryId      : %hu\n", *ret, data.CountryId);
            ret = cString::sprintf("%sSatId          : %hu\n", *ret, data.SatId);
            ret = cString::sprintf("%sscanflags      : %u\n",  *ret, data.scanflags);
            ret = cString::sprintf("%sATSC_type      : %hu\n", *ret, data.ATSC_type);
            return ret;
            }
         else {
            free(s);
            return cString::sprintf("This plugin version doesnt support requested api.");
            }     
         }
     else {
         // "wirbelscan_SetSetup#<VERSION>"
         asprintf(&s, "%sSet%s", SPlugin, SSetup);
         if (wirbelscan->Service(s, ScanSetup) == true) {
            free(s);
            ret = cString::sprintf("--new settings--\n");
            ret = cString::sprintf("%sverbosity      : %hu\n", *ret, ScanSetup->verbosity);
            ret = cString::sprintf("%slogFile        : %hu\n", *ret, ScanSetup->logFile);
            ret = cString::sprintf("%sDVB_type       : %hu\n", *ret, ScanSetup->DVB_Type);
            ret = cString::sprintf("%sDVBT_Inversion : %hu\n", *ret, ScanSetup->DVBT_Inversion);
            ret = cString::sprintf("%sDVBC_Inversion : %hu\n", *ret, ScanSetup->DVBC_Inversion);
            ret = cString::sprintf("%sDVBC_Symbolrate: %hu\n", *ret, ScanSetup->DVBC_Symbolrate);
            ret = cString::sprintf("%sDVBC_QAM       : %hu\n", *ret, ScanSetup->DVBC_QAM);
            ret = cString::sprintf("%sCountryId      : %hu\n", *ret, ScanSetup->CountryId);
            ret = cString::sprintf("%sSatId          : %hu\n", *ret, ScanSetup->SatId);
            ret = cString::sprintf("%sscanflags      : %u\n",  *ret, ScanSetup->scanflags);
            ret = cString::sprintf("%sATSC_type      : %hu\n", *ret, ScanSetup->ATSC_type);
            return ret;
            }
         else {
            free(s);
            return cString::sprintf("This plugin version doesnt support requested api.");
            }
         }
     }
 else
     return cString::sprintf("wirbelscan not found.\n");
}


cString cPluginServdemo::SVDRPCommand(const char *Command, const char *Option, int &ReplyCode)
{
  cWirbelscanCmd cmd = { CmdStopScan, false };
  
  if (!strcasecmp(Command,"START")) {
      cmd.cmd = CmdStartScan;
      SendCmd(&cmd);
      return cmd.replycode?"scan started":"Could not start scan.";
      }
  else if(!strcasecmp(Command,"STOP")) {
      cmd.cmd = CmdStopScan;
      SendCmd(&cmd);
      return cmd.replycode?"scan stopped":"Could not stop scan.";
      }
  else if(!strcasecmp(Command,"STORE")) {
      cmd.cmd = CmdStore;
      SendCmd(&cmd);
      return cmd.replycode?"store triggered":"Could not trigger store.";
      }
  else if(!strcasecmp(Command,"QUERY")) {
      cWirbelscanInfo info;
      if (QueryVersions(&info))
          return cString::sprintf(
                 "wirbelscan-%s\ncommand api '%s'\nstatus api  '%s'\nsetup api   '%s'\ncountry api '%s'\nsat api     '%s'\n",
                  info.PluginVersion,
                  info.CommandVersion,
                  info.StatusVersion,
                  info.SetupVersion,
                  info.CountryVersion,
                  info.SatVersion);
      else
          return "Could not query version.";
      }
  else if(!strcasecmp(Command,"LSTS")) {
      return ListSatellites();
      }
  else if(!strcasecmp(Command,"LSTC")) {
      return ListCountries();
      }
  else if(!strcasecmp(Command,"STAT")) {
      return Status();
      }
  else if(!strcasecmp(Command,"GETS")) {
      return setup(NULL);
      }
  else if(!strcasecmp(Command,"SETS")) {
      cWirbelscanScanSetup d;
      if (11 != sscanf(Option, "%hu:%hu:%hu:%hu:%hu:%hu:%hu:%hu:%hu:%u:%hu",
              &d.verbosity, &d.logFile,
              &d.DVB_Type,
              &d.DVBT_Inversion, &d.DVBC_Inversion,
              &d.DVBC_Symbolrate, &d.DVBC_QAM,
              &d.CountryId, &d.SatId,
              &d.scanflags, &d.ATSC_type))
         {
         ReplyCode = 501;
         return "couldnt parse setup string.";
         }
      return setup(&d);
      }
  return NULL;
}

VDRPLUGINCREATOR(cPluginServdemo); // Don't touch this!
